<?php
// $Id: ec_charge.admin.inc,v 1.1.2.6 2007/12/19 06:28:55 brmassa Exp $

/**
 * @author e-Commerce Dev Team
 * @file ec_charge.admin.inc
 * All admin function is located here to avoid useless module footprint
 */

define('EC_CHARGE_CHARGE', 1);
define('EC_CHARGE_CHARGE_PCT_SUBTOTAL', 2);
define('EC_CHARGE_CHARGE_PCT_ITEMTOTAL', 3);

/**
 * Creating/editing charge form
 */
function _ec_charge_admin_form() {
  $charge_methods = _ec_charge_charge_methods();

  switch (arg(3)) {
    case 'add':
      drupal_set_title(t('New charge'));
      $op = 'add';
      $chid = 0;
      $provider = $_GET['provider'];
      $method = $_GET['method'];
      break;
    case 'edit':
      $op = 'edit';
      $chid = arg(4);
      include_once drupal_get_path('module', 'ec_charge') .'/ec_charge.inc';
      $charge = _ec_charge_load_charges($chid);
      $provider = $charge->provider; // convenience
      $method = $charge->method; // convenience
      drupal_set_title(t('Edit') .' '. check_plain($charge->display));
      break;
  }

  $form = array();
  $form['module'] = array(
    '#type'           => 'fieldset',
    '#title'          => 'Flexicharge defaults',
    '#description'    => 'Charge description goes here',
    '#collapsible'    => true,
    '#collapsed'      => false,
    '#tree'           => true
  );
  $form['module']['chid'] = array(
    '#type'           => 'value',
    '#value'          => $chid
  );
  $form['module']['provider'] = array(
    '#type'           => 'value',
    '#value'          => $provider
  );
  $form['module']['method'] = array(
    '#type'           => 'value',
    '#value'          => $method
  );
  $form['module']['display'] = array(
    '#type'           => 'textfield',
    '#required'       => true,
    '#title'          => t('Display label'),
    '#description'    => t('This is the description for the review screen and invoice.'),
    '#default_value'  => ($chid) ? $charge->display : $charge_methods[$provider][$method]['#title'],
    '#size' => 20,
  );
  $operators[0] = '--';
  $operators = _ec_charge_operators('list');

  $form['module']['elements'] = array();

  $form['module']['operator'] = array(
    '#type' => 'select',
    '#title' => t('Operator'),
    '#default_value' => ($chid) ? $charge->operator : EC_CHARGE_ADD,
    '#options' => $operators,
    );
  $form['module']['rate'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Charge rate'),
    '#default_value' => ($chid) ? $charge->rate : 0,
    '#description'   => t('Use positive or negative values. You can specify percent like 10 or -20, use the operator to specify percentage.'),
    '#size'          => 8,
  );
  $form['module']['position'] = array(
    '#type'          => 'textfield',
    '#required'      => true,
    '#title'         => t('Calculation order'),
    '#description'   => t('Position of this charge in the ec_charge calculations.'),
    '#default_value' => ($chid) ? $charge->position : 1,
    '#size'          => 5,
  );
  $form['module']['subtotal_before'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display a subtotal before?'),
    '#default_value' => ($chid) ? $charge->subtotal_before : 0,
  );
  $form['module']['subtotal_after'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display a subtotal after?'),
    '#default_value' => ($chid) ? $charge->subtotal_after : 0,
  );
  $form['module']['already_added'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display this amount as \'included\', but don\'t add it to the subtotal.'),
    '#default_value' => ($chid) ? $charge->already_added : 0,
  );
  $form['module']['hide_if_zero'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Suppress this row in the cart if the total charge = 0.'),
    '#default_value' => ($chid) ? $charge->hide_if_zero : 0,
  );
  $ptypes = product_get_ptypes('names');
  $form['module']['ptypes'] = array(
    '#type'          => 'select',
    '#title'         => t('Product types'),
    '#options'       => $ptypes,
    '#multiple'      => true,
    '#default_value' => empty($charge->ptypes) ? 0 : $charge->ptypes,
    '#description'   => 'Please choose the products that this charge applies to. No selection indicates all products. Charge will apply only once, if any product in the cart matches this list.',
  );
  $all_roles = user_roles();
  $form['module']['roles'] = array(
    '#type'          => 'select',
    '#title'         => t('Roles'),
    '#options'       => $all_roles,
    '#multiple'      => true,
    '#default_value' => (($charge->roles) ? $charge->roles : 0),
    '#description'   => 'Please select roles that this charge applies to. No selection indicates all roles. Charge will apply only once, if user is in any selected role.',
  );

  // Get the form elements required by the charge provider
  $f = $provider .'_ec_charge_form_elements';
  if (function_exists($f)) {
    //$form passed by array
    $f($form, $op, $method);
    $form['elements']['#tree'] = true;
  }

  // If there are custom form elements - apply them from the database values
  if (arg(3) == 'edit' && is_array($charge->settings)) {
    foreach ($charge->settings as $e => $s) {
      foreach ($form['elements'] AS $f => $fieldset) {
        if (isset($form['elements'][$f][$e])) {
          $form['elements'][$f][$e]['#default_value'] = $s;
        }
      }
    }
  }
  $form['submit'] = array(
    '#type'          => 'submit',
    '#value'         => t('Save charge'),
  );

  return $form;
}

/**
 * Implementation of hook_submit().
 *
 * Creating/editing charge form
 */
function _ec_charge_admin_form_submit($form_id, $form) {
  // Get the form elements required by the charge provider
  $f = $provider .'_ec_charge_form_validate';
  if (function_exists($f)) {
    $form = $f($form_id, $form);
  }

  $settings = array();
  if (isset($form['elements'])) {
    foreach ($form['elements'] AS $fieldset) {
      $settings = array_merge($fieldset, $settings);
    }
    $settings = serialize($settings);
  }
  else {
    $settings = '';
  }

  $ptypes = serialize($form['module']['ptypes']);
  $roles = serialize($form['module']['roles']);

  if (!empty($form['module']['chid'])) {
    db_query("UPDATE {ec_charge} SET display = '%s', operator = %d, rate = %f,
      position = %d, subtotal_before = %d, subtotal_after = %d, already_added = %d,
      hide_if_zero = %d, ptypes = '%s', roles = '%s', settings = '%s' WHERE chid = %d",
      $form['module']['display'], $form['module']['operator'],
      $form['module']['rate'], $form['module']['position'],
      $form['module']['subtotal_before'], $form['module']['subtotal_after'],
      $form['module']['already_added'], $form['module']['hide_if_zero'],
      $ptypes, $roles, $settings, $form['module']['chid']);
  }
  else {
    db_query("INSERT INTO {ec_charge} ( provider, method, display, operator,
      rate, position, subtotal_before, subtotal_after, already_added, hide_if_zero,
      ptypes, roles, settings) VALUES ('%s', '%s', '%s', %d, %f, %d, %d, %d, %d,
      %d, '%s', '%s', '%s')", $form['module']['provider'], $form['module']['method'],
      $form['module']['display'], $form['module']['operator'],
      $form['module']['rate'], $form['module']['position'],
      $form['module']['subtotal_before'], $form['module']['subtotal_after'],
      $form['module']['already_added'], $form['module']['hide_if_zero'],
      $ptypes, $roles, $settings);
  }

  drupal_set_message(t('The charge has been saved.'));
  drupal_goto('admin/ecsettings/ec_charge');
}

/**
 * Delete a charge
 */
function _ec_charge_charge_delete() {
  db_query('DELETE FROM {ec_charge} WHERE chid = %d', arg(4));
  drupal_goto('admin/ecsettings/ec_charge');
}

/**
 * This function mimics shipcalc_shipping_methods() and for cross-reference
 * purposes I have kept it mostly the same. In shipcalc, a 'partner' is UPS.
 * USPS etc, in ec_charge, a 'partner' is actually a collection of one or
 * more charge types.
 *
 * @return
 *   Array, ec_charge methods.
 */
function _ec_charge_charge_methods() {
  include_once drupal_get_path('module', 'ec_charge') .'/ec_charge.inc';
  $methods = array();
  $partners = _ec_charge_partners();
  foreach ($partners as $partner) {
    $charge_methods = $partner->name .'_charge_methods';
    if (function_exists($charge_methods)) {
      $methods = array_merge($methods, $charge_methods(NULL));
    }
  }
  return $methods;
}

/**
 * List all charges and new charge options
 */
function _ec_charge_overview() {
  // First we display a list of currently configured charges.
  $operators = _ec_charge_operators('list');
  $charge_list = array();

  // Check if there is any created charge
  include_once drupal_get_path('module', 'ec_charge') .'/ec_charge.inc';
  if ($charges = _ec_charge_load_charges()) {
    foreach ($charges AS $charge) {
      $row['name']   = l($charge->display, 'admin/ecsettings/ec_charge/edit/'. $charge->chid);
      $row['type']   = $charge->provider;
      $row['weight'] = $charge->position;
      $row['delete'] = l('delete', 'admin/ecsettings/ec_charge/delete/'. $charge->chid);
      array_push($charge_list, $row);
    }
    $output = theme('fieldset', array(
      '#title'       => 'Currently configured charges',
      '#children'    => theme('table', array('Name', 'Type',
        'Weight', 'Operations'), $charge_list),
      '#collapsible' => TRUE,
      '#collapsed'   => FALSE)
    );
  }
  else {
    $output = '<p>'. t('There is no charges yet.') .'</p>';
  }

  // Second we add a fieldset containing links to create new charges.
  $charge_methods = _ec_charge_charge_methods();
  foreach ($charge_methods as $provider => $methods) {
    foreach (element_children($methods) as $method) {
      $charge_menu[] = l($methods[$method]['#title'],
        'admin/ecsettings/ec_charge/add', NULL,
        "provider=$provider&method=$method");
    }
  }

  $output .= theme('fieldset', array(
    '#title'    => 'Add a new charge',
    '#children' => theme('item_list', $charge_menu)));
  return $output;
}

/**
 * List all possible operators
 */
function _ec_charge_operators($op = 'list') {
  switch ($op) {
    case 'list':
      return array(
        EC_CHARGE_CHARGE                => 'Simple +/-',
        EC_CHARGE_CHARGE_PCT_SUBTOTAL   => '% of Subtotal',
        EC_CHARGE_CHARGE_PCT_ITEMTOTAL  => '% of Item Total',
        );
  }
}
