<?php
// $Id: ec_receipt.php,v 1.1.2.2 2007/11/26 05:33:42 gordon Exp $

/**
 * @file
 * These are hooks which are called using ec_receipt_invoke() or 
 * ec_receipt_alloc_invoke()
 */

/**
 * @ingroup hooks
 * @{
 */

/**
 * @defgroup "e-Commerce Receipts" "e-Commerce Receipts"
 * @{
 * The Drupal e-Commerce Receipts system
 *
 * Welcome to the Drupal developers documentation for Drupal e-Commerce 
 * Receipts package.
 */

/**
 * Register payments gateways to the e-Commerce Receipts system
 *
 * @return
 *  Return an array or receipt info blocks to the system.
 */
function hook_receipt_info() {
  // Because you can create a module what provides several
  // payment gateways, we ask for a array containing
  // info about each of them.
  return array(
    'cod' => array(
      // Description
      'name'                 => t('Gateway example'),
      'description'          => t('Clients pay when they receive their products.'),

      // (Optional). The payment icon
      'icon' => array(
        'src' => drupal_get_path('module', 'example') .'/example.gif',
        'attributes' => array(
          'title' => t('Example Logo'),
          'class' => 'example-logo',
        ),
      ),

      // List only the currencies supported by your
      // payment gateway
      'currencies_supported' => array('AUD', 'USD'),

      // Remember to set the 'module' attribute
      // differently to each payment gateway
      'module'               => array('example'),

      // (Optional). All money operations that this method allow.
      // For payment gateways, use "allow_payments".
      'allow_payments'       => TRUE,
      'allow_admin_payments' => TRUE,
      'allow_refunds'        => TRUE,
      'allow_payto'          => TRUE,
      'allow_recurring'      => TRUE,

      // (Optional). Extra third party requirements. You should include all
      // restrictions and notices.
      'gateway_requirements' => NULL,
    ),
  );
}

/**
 * Load additional receipt information from the database.
 *
 * @param $receipt
 *  Receipt record which this hook is meant to load the additional 
 *  information.
 * 
 * @return
 *  Return an array of additional information to be included into the 
 *  receipt before it is returned to the system.
 */
function hook_receipt_load($receipt) {
}

/**
 * Save additional information about the receipt to the database.
 *
 * @param $receipt
 *  The receipt object which is going to be saved.
 */
function hook_receipt_save($receipt) {
}

/**
 * Build a form for collecting the payment information.
 *
 * This will generally be used on the checkout review page which will 
 * collect the information that is required to charge the customer.
 *
 * @param $type
 *  The allocation type which corresponds to the $object which can be past 
 *  to the e-Commerce Receipt allocation interface.
 *
 * @param $object
 *  This is a data type which can pass the information to the allocation 
 *  interface to reteive data from the object.
 *
 * @return
 *  Return the form which can be displayed.
 */
function hook_receipt_payment_form($type, $object) {
}

/**
 * Using the information past from the payment form the system will process 
 * the payment and update the receipt.
 *
 * @param $receipt
 *  A blank receipt is past to the system for the payment to be connected 
 *  to. The identifier for the payment is against the receipt and not the 
 *  transactions. At this stage the transaction may or may not have been 
 *  created, and if there is a crash the receipt can be tracked to the 
 *  customer.
 *
 * @param $type
 *  The allocation type which corresponds to the $object which can be past 
 *  to the e-Commerce Receipt allocation interface.
 *
 * @param $object
 *  This is a data type which can pass the information to the allocation 
 *  interface to reteive data from the object.
 *
 */
function hook_receipt_process_payment($receipt, $atype, $object) {
}

/**
 * @param $receipt
 *  A blank receipt is past to the system for the payment to be connected 
 *  to. The identifier for the payment is against the receipt and not the 
 *  transactions. 
 *
 * @param $type
 *  The allocation type which corresponds to the $object which can be past 
 *  to the e-Commerce Receipt allocation interface.
 *
 * @param $object
 *  This is a data type which can pass the information to the allocation 
 *  interface to reteive data from the object.
 *
 * @return
 *  Return the url which is past to drupal_goto() to be devirted to the 
 *  hosted payment gateway system.
 */
function hook_receipt_payment_url($receipt, $atype, $object) {
}

/**
 * @} End of "defgroup e-Commerce Receipts"
 */

/**
 * @} End of "ingroup hooks".
 */
