<?php
// $Id: correios.inc,v 1.1.2.4 2007/12/19 06:27:24 brmassa Exp $
/**
 * @author Bruno Massa
 * @file
 * Functions to communicate with Correios API.
 *
 * @todo ensure that the weight is in kilograms
 */

/**
 * Implementation of hook_get_rates().
 *
 * Depending on what is the origin and destiny, and the weight of the
 * product, Correios charges diferently.
 */
function correios_get_rates(&$txn) {
  // Services and their codes
  $services = array();
  $services['PAC']            = '41106';
  $services['SEDEX']          = '40010';
  $services['SEDEX 10']       = '40215';
  $services['SEDEX Hoje']     = '40290';
  $services['SEDEX a Cobrar'] = '40045';
  $services['e-SEDEX']        = '81019';
  $services['Malote']         = '44105';

  // Fulfillment center address (should be in brazil)
  $fulfillmentcenter = shipping_fulfillmentcenter_default();
  if ($fulfillmentcenter['country'] != 'br' or empty($fulfillmentcenter['code'])) {
    return;
  }

  // Check which shipping methods from Correios are enabled
  foreach ($txn->items as $nid => $item) {
    if ($methods = shipping_get_methods(NULL, $item->ptype)) {
      foreach ($methods as $partner => $method) {
        if ($partner == 'correios') {
          foreach (element_children($method) as $code) {
              $enabled[$code] = TRUE;
          }
        }
      }
    }
  }

  // If no method is enabled, exit
  if (empty($enabled)) {
    return;
  }

  // Sum all products' weights
  $weight = 0;
  if (is_array($txn->items) && $txn->items != array()) {
    foreach ($txn->items as $item) {
      // Load product_weight into $item.
      shipping_nodeapi($item, 'load', null);
      if (!empty($item->product_attributes['weight'])) {
        $weight += $item->product_attributes['weight'] * $item->qty;
      }
    }
  }

  // Calculate the shipping cost of each of
  // shipping methods
  while (list($service_name, $code) = each($services)) {
    if (!empty($enabled[$service_name])) {
      // GET options
      $options = array(
        'Servico'     => $code,
        'cepOrigem'   => $fulfillmentcenter['code'],
        'cepDestino'  => $txn->address['shipping']->zip,
        'peso'        => $weight,
      );
      $return = drupal_http_request(url('http://www.correios.com.br/internet/encomendas/precos/calculo.cfm',
        drupal_query_string_encode($options)));

      if (empty($return->error)) {
        preg_match('/Tarifa=(.*)\&erro=/', $return->data, $charge);
        if (!empty($charge[1])) {
          $rates[] = array(
            '#service'    => 'correios',
            '#key'        => $service_name,
            '#cost'       => $charge[1],
            '#currency'   => 'BRL',
            '#method'     => $service_name
          );
        }
      }
    }
  }
  if (!empty($rates)) {
    return $rates;
  }
}

/**
 * Implementation of hook_product_attributes().
 *
 * Update the product form with fields that we need.  It is possible for
 * multiple carriers to define the same field -- that is fine.  So long as
 * the field as the same name (e.g. 'weight'), it will only be displayed
 * once, and the data will be saved and restored for use by all shipping
 * partners that define it.
 */
function correios_product_attributes($form) {
  include_once drupal_get_path('module', 'shipping') .'/shipping.dimensions.inc';
  $fields = array('weight');
  return _shipping_dimensions_form($form, $fields);
}

/**
 * Implementation of hook_shipping_methods().
 *
 * List all Correios shipping methods.
 */
function correios_shipping_methods() {
  $methods = array();
  $methods['correios'] = array(
    '#title'       => t('Correios'),
    '#description' => t('Companhia de Correios e Telégrafos')
  );
  $methods['correios']['PAC'] = array(
    '#title'       => t('PAC')
  );
  $methods['correios']['SEDEX'] = array(
    '#title'       => t('SEDEX')
  );
  $methods['correios']['SEDEX 10'] = array(
    '#title'       => t('SEDEX 10')
  );
  $methods['correios']['SEDEX a Cobrar'] = array(
    '#title'       => t('SEDEX a Cobrar')
  );
  $methods['correios']['SEDEX HOJE'] = array(
    '#title'       => t('SEDEX HOJE')
  );
  $methods['correios']['e-SEDEX'] = array(
    '#title'       => t('e-SEDEX')
  );
  $methods['correios']['Malote'] = array(
    '#title'       => t('Malote')
  );
  return $methods;
}
