<?php
// $Id: subproducts.inc,v 1.3.2.1.4.4.2.11 2007/11/29 20:35:23 brmassa Exp $
/**
 * @author Bruno Massa http://drupal.org/user/67164
 * @file subproducts.inc
 * Automaticly create subproducts based on attributes
 *
 * Imagine you want to sell shirts with diferent sizes and colors.
 * With normal products module, you should create one by one.
 * With Subproducts, you create first each variation (size and color) and
 * the module creates for you all possible combinations! And for your customers,
 * its also easy to choose theirs product combination.
 */

/**
 * Get all attributes
 *
 * @param $id
 *   Number, parent variation ID
 * @param $type
 *   String, the search type
 *    baseproduct = get all attributes from a given parent subproduct
 *    product     = get all attributes from a given subproduct
 * @param $pager
 *   Number, how many variations show be selected
 * @return
 *   Object, all attributes and theirs fields or null
 */
function _subproducts_attribute_get($id = null, $type = null, $pager = null) {
  static $variations = array(); // try to cache the results
  if (!isset($variations[$id][$type][$pager])) { // results is cached
    $sql_join[]   = ' INNER JOIN {ec_sp_variation} sv ON sv.tid = td.tid
                      LEFT JOIN {term_node} tn ON tn.tid = td.tid';
    $sql_where[]  = ' td.vid = %d ';
    $sql_args[]   = variable_get('ec_sp_taxomony', 0);
    if     ($type == 'baseproduct' and !empty($id)) {
      array_push($sql_join, ' INNER JOIN {ec_sp_bpattribute} a ON a.nid = tn.nid ');
      array_push($sql_where, ' AND a.pnid  = %d');
      array_push($sql_args, $id);
    }
    elseif ($type == 'product' and !empty($id)) {
      array_push($sql_join, ' LEFT JOIN {ec_sp_pattribute} a ON a.nid = tn.nid');
      array_push($sql_where, ' AND a.pnid  = %d');
      array_push($sql_args, $id);
    }
    $sql_final = 'SELECT td.*, sv.*, tn.nid
      FROM {term_data} td '. implode('', $sql_join) .
      ' WHERE '. implode('', $sql_where) .' ORDER BY weight ASC';
    $sql_final = db_rewrite_sql($sql_final);
    if (empty($pager)) {
      $result = db_query($sql_final, $sql_args);
    }
    else {
      $result = pager_query($sql_final, $pager, 0, null, $sql_args);
    }
    $variations[$id][$type][$pager] = array();
    while ($variation = db_fetch_array($result)) {
      if (empty($variations[$id][$type][$pager][$variation['tid']])) {
        $variations[$id][$type][$pager][$variation['tid']] = $variation;
      }
      if (!empty($variation['nid'])) {
        $variations[$id][$type][$pager][$variation['tid']]['attr'][$variation['nid']] =
          node_load($variation['nid']);
      }
    }
  }
  return $variations[$id][$type][$pager];
}

/**
 * Implementation of hook_cart_add_item().
 *
 *  Add the combinated product to cart
 *
 * @param &$node
 *   Object, a node object representing a parent product
 * @param $qty
 *   Number, the quantity of items
 * @param $data
 *   Object, the subproduct attributes
 * @return
 *   Boolean, indicating success or failure in setting a subproduct
 */
function _subproducts_feature_pproduct_cart_validate_item(&$node, $qty, $data) {
  if (!product_feature_exists($node->ptype, 'pproduct') or !empty($node->pproduct)) {
    return true;
  }

  // When the "Update price" button is pressed, it means that
  // Javascript is not working
  $priceupdate = array();
  if (!empty($_POST['priceupdate']) and !empty($_POST['variations']) ) {
    while (list($variation, $attribute) = each($_POST['variations'])) {
      array_push($priceupdate, $variation .','. $attribute);
    }
    drupal_goto($_REQUEST['q'], 'priceupdate='. implode(';', $priceupdate));
  }

  // Create the subproduct if it doesnt exist
  $node_old = subproducts_php_clone($node);
  $sql_terms = $sql_args = $sql_select = $sql_where = $tag = $forbidden = array();
  $node->stock = -1;
  $node->title = empty($node->sku_title) ? $node->title : $node->sku;
  if (!empty($_POST['variations'])) {
    foreach ($_POST['variations'] as $tid => $nid) {
      $attribute = $node->sp_attributes[$tid]['attr'][$nid];
      array_push($sql_terms, '(%d,%d)');
      array_push($sql_args, 'XX');
      array_push($sql_args, $nid);
      array_push($sql_select, ' INNER JOIN {ec_sp_pattribute} a'. $nid .' ON a'.
        $nid .'.pnid = p.nid ');
      array_push($sql_where, ' AND a'. $nid .'.nid = '. $nid);
      $node->price += $attribute->price;
      while (list(, $term) = each($attribute->taxonomy)) {
        if ($term->vid == variable_get('ec_sp_taxomony', 0) + 1) {
          $tag[$term->name] = 1;
        }
        if ($term->vid == variable_get('ec_sp_taxomony', 0) + 2) {
          $forbidden[$term->name] = 1;
        }
      }
      if (product_feature_exists($node->ptype, 'shippable') and (
          ($stock > $attribute->stock and $attribute->stock != -1) or
          ($stock < $attribute->stock and $stock == -1))) {
        $node->stock = $attribute->stock;
      }

      // Create the new product's title
      if (empty($node->sku_title)) {
        $node->title .= ' '. $attribute->title;
      }
      else {
        $node->title .= $attribute->sku;
      }
    }
  }

  // If this combination is forbidden, cancel process
  while (list($forbid) = each($forbidden)) {
    if (!empty($tag[$forbid])) {
      drupal_set_message(t('This combination is not possible.'), 'error');
      return false;
    }
  }

  // Search for a parent product with such attributes
  $result = 'SELECT p.nid FROM {ec_sp_pproduct} p '. implode('', $sql_select) .
    ' WHERE p.pproduct = %d '. implode('', $sql_where);
  $result = db_result(db_query($result, $node->nid));
  if (empty($result)) {
    $node->pproduct = $node->nid;
    unset($node->comment);
    unset($node->voting);
    unset($node->path);
    unset($node->nid);
    unset($node->sticky);
    unset($node->promote);
    node_save($node);
    while (list($index, $value) = each($sql_args)) {
      if ($value == 'XX') {
        $sql_args[$index] = $node->nid;
      }
    }
    db_query('INSERT INTO {ec_sp_pattribute} (pnid, nid) VALUES '.
      implode(',', $sql_terms), $sql_args);
  }
  else {
    $node = node_load($result);
  }
  return true;
}

/**
 * Implementation of hook_form_alter().
 */
function _subproducts_form_alter($form_id, &$form) {
  if (!empty($form['type']) and $form_id == $form['type']['#value'] .'_node_form') {
    if (product_feature_exists($form['#node']->ptype, 'attribute')) {
      $form['title']['#title'] =  t('Attribute');
      $form['attr_weight'] = array(
        '#type'           => 'weight',
        '#title'          => t('Weight'),
        '#default_value'  => isset($node->attr_weight) ? $node->attr_weight : 0);
      if (!db_num_rows(db_query('SELECT tid FROM {term_data} WHERE vid = %d',
          variable_get('ec_sp_taxomony', 0)))) {
        drupal_set_message(t('You cannot create an attribute without a variation. !Create_variations.',
          array('!Create_variations' => l(t('Create a variation first'),
          'admin/content/taxonomy/'. variable_get('ec_sp_taxomony', 0) .'/add/term',
          null, 'destination=node/add/product/'. $form['#node']->ptype))), 'error');
        $form['submit']['#disabled'] = true;
      }
    }
    else {
      unset($form['taxonomy'][variable_get('ec_sp_taxomony', 0)]);
      unset($form['taxonomy']['tags'][variable_get('ec_sp_taxomony', 0) + 1]);
      unset($form['taxonomy']['tags'][variable_get('ec_sp_taxomony', 0) + 2]);

      // Remove the fieldset grouping categories
      if (count(element_children($form['taxonomy'])) == 1) {
        unset($form['taxonomy']['#type']);
      }
    }
  }
  elseif ($form_id == 'cart_view_form') {
    while (list($product) = each($form['products'])) {
      if (!is_numeric($product)) {
        continue;
      }
      $attributes = '';
      $variations = subproducts_attribute_get($product, 'product');
      while (list($vid, $variation) = each($variations)) {
        while (list(, $attribute) = each($variation['attr'])) {
          if (!empty($form['#parameters'][1][$product]->variations) and
              $attribute->nid == $form['#parameters'][1][$product]->variations[$vid]) {
            $form['products'][$product]['title']['attributes'][] = array(
              '#title'  => $variation['name'],
              '#theme'  => 'subproducts_cart_view',
              '#type'   => 'item',
              '#value'  => $attribute->title
            );
          }
        }
      }
    }
  }
  elseif ($form_id == 'taxonomy_form_term' and $form['vid']['#value'] == variable_get('ec_sp_taxomony', 0)) {
    if (isset($form['tid'])) {
      $term = db_fetch_array(db_query('SELECT * FROM {ec_sp_variation} WHERE tid = %d', $form['tid']['#value']));
    }
      $form['listtype'] = array(
        '#type' => 'select',
        '#title' => t('List type'),
        '#default_value' => isset($form['tid']) ? $term['listtype'] : 0,
        '#options' => array(t('Radio List'), t('Drop-Down List')),
        '#description' => t('How your attributes should be presented'),
      );
      $form['showprice'] = array(
        '#type' => 'select',
        '#title' => t('Show surcharge'),
        '#default_value' => isset($form['tid']) ? $term['showprice'] : 0,
        '#options' => array(t('Yes'), t('No')),
        '#description' => t('Show surcharge after the attribute name, like "Iron (+$10.00)"'),
      );
    $form['submit']['#weight'] = 10;
    if (!empty($form['delete'])) {
      $form['delete']['#weight'] = 10;
    }
  }
  elseif ($form_id == 'taxonomy_form_vocabulary' and (
      $form['vid']['#value'] == variable_get('ec_sp_taxomony', null) or
      $form['vid']['#value'] == variable_get('ec_sp_taxomony', 0) + 1 or
      $form['vid']['#value'] == variable_get('ec_sp_taxomony', 0) + 2)) {

    // Select all node types that have "attribute" feature
    foreach (node_get_types() as $type) {
      if (variable_get('ec_product_convert_'. $type->type,
        variable_get('ec_product_convert', 0)) and $ptypes = variable_get('ec_product_ptypes_'.
        $type->type, array_keys(product_get_ptypes('names')))) {
        foreach ($ptypes as $ptype) {
          if (product_feature_exists($ptype, 'attribute')) {
            $types[$type->type] = $type->type;
            break;
          }
        }
      }
    }
    $form['nodes']['#value']    = $types;
    $form['nodes']['#type']     = 'value';

    if ($form['vid']['#value'] == variable_get('ec_sp_taxomony', 0) + 1 or
        $form['vid']['#value'] == variable_get('ec_sp_taxomony', 0) + 2) {
      $form['tags']['#value'] = true;
      $form['tags']['#disabled'] = true;
    }
    else {
      unset($form['tags']);
    }
    unset($form['multiple']);
    unset($form['hierarchy']);
    unset($form['relations']);
    unset($form['required']);
    unset($form['delete']);
  }
}

/**
 * Show all attributes from a given product
 *
 * @param &$node
 *   Object, the node/product
 * @return
 *   String, a HTML form with all attributes
 */
function subproducts_pproduct_view(&$node) {
  $listtypes = array('radios', 'select');
  $price = $current_variation = null;

  // When your dont have Javascript enabled and the priceupdate
  // values come from URL
  if (!empty($_GET['priceupdate'])) {
    $variations = explode(';', $_GET['priceupdate']);
    while (list(, $variation) = each($variations)) {
      if (empty($variation)) {
        break;
      }
      $variation2 = explode(',', $variation);
      $priceupdate[$variation2[0]] = $variation2[1];
    }
  }

  while (list($tid, $variation) = each($node->sp_attributes)) {
    while (list($nid, $attr) = each($variation['attr'])) {
      if (empty($json_attr[$tid]['checked'])) {
        if (empty($priceupdate)) {
        $json_attr[$tid]['checked'] = empty($node->default_selection[$tid]) ?
          $nid : $node->default_selection[$tid];
        }
        else {
            $json_attr[$tid]['checked'] = $priceupdate[$tid];
        }
      }

      if ($json_attr[$tid]['checked'] == $nid) {
        $price += $attr->price;
        $variation['attribute_description'] = empty($attr->description) ?
          '' : $attr->description;
      }

      $options[$tid][$nid] = theme('subproducts_attribute_price', $attr->price, $attr->title);
      $json_attr[$tid][$nid]['price'] = $attr->price;
      $json_attr[$tid][$nid]['category'] = $attr->category;
      $json_attr[$tid][$nid]['forbidden'] = $attr->forbidden;
      $json_attr[$tid][$nid]['description'] = empty($attr->body) ?
        '' : check_markup($attr->body, $attr->format);

      // Add stock values in case its a shippable attribute
      if (product_feature_exists($attr->ptype, 'shippable')) {
        $json_attr[$tid][$nid]['stock'] = (int) $attr->stock;
      }
      else {
        $json_attr[$tid][$nid]['stock'] = -1;
      }
    }
    $form['title'][$tid] = array(
      '#type' => 'markup',
      '#value' =>  $variation['name']
    );
    $form['vardesc'][$tid] = array(
      '#type' => 'markup',
      '#value' => $variation['description']
    );
    $form['variations'][$tid] = array(
      '#attributes' => array('class' => 'attr'),
      '#default_value' => $json_attr[$tid]['checked'],
      '#options' => $options[$tid],
      '#type' => $listtypes[$variation['listtype']],
    );
    $form['description'][$tid] = array(
      '#type' => 'markup' ,
      '#value' => check_markup($variation['attribute_description'])
    );
  }

  // Javascript payment format variables
  $payment_format['dpos'] = variable_get('payment_decimal_places', 2);  // decimal places
  $payment_format['decimal'] = variable_get('payment_decimal', '.');    // decimal symbol
  $payment_format['thousand'] = variable_get('payment_thousands', ','); // thousand symbol
  $payment_format['symbol'] = variable_get('payment_symbol', '$');      // current symbol
  $payment_format['spos'] = variable_get('payment_symbol_position', 1); // symbol position
  drupal_add_js(array('FormatPrice' => $payment_format), 'setting');
  drupal_add_js(array('price' => (float) $node->price), 'setting');

  // Translation on Javascript
  drupal_add_js(array('messages' => array('price' => t('Price'),
    'sold_out' => t('Sold out'),
    'forbidden' => t('This combination is not possible.'))), 'setting');
  drupal_add_js(array('attr' => $json_attr), 'setting');

  $form['newprice'] = array(
    '#prefix' => '<div id="price">',
    '#type' => 'item',
    '#title' => t('Price'),
    '#value' => format_currency($node->price + $price)
  );
  $form['priceupdate'] = array(
    '#name' => 'priceupdate',
    '#type' => 'button',
    '#value' => t('Update Price')
  );
  $form['add-to-cart'] = array(
    '#attributes' => array('class' => 'addtocart'),
    '#suffix' => '</div>',
    '#type' => 'submit',
    '#value' => t('Add to Cart')
  );
  $form['#method'] = 'post';
  $form['#action'] = url('cart/add/'. $node->nid, 'destination=node/'. $node->nid);
  $form['#tree'] = true;
  return $form;
}
